/******************************************************************************
 * Spine Runtimes License Agreement
 * Last updated January 1, 2020. Replaces all prior versions.
 *
 * Copyright (c) 2013-2020, Esoteric Software LLC
 *
 * Integration of the Spine Runtimes into software or otherwise creating
 * derivative works of the Spine Runtimes is permitted under the terms and
 * conditions of Section 2 of the Spine Editor License Agreement:
 * http://esotericsoftware.com/spine-editor-license
 *
 * Otherwise, it is permitted to integrate the Spine Runtimes into software
 * or otherwise create derivative works of the Spine Runtimes (collectively,
 * "Products"), provided that each user of the Products must obtain their own
 * Spine Editor license and redistribution of the Products in any form must
 * include this license and copyright notice.
 *
 * THE SPINE RUNTIMES ARE PROVIDED BY ESOTERIC SOFTWARE LLC "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ESOTERIC SOFTWARE LLC BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES,
 * BUSINESS INTERRUPTION, OR LOSS OF USE, DATA, OR PROFITS) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THE SPINE RUNTIMES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

package spine.flash {
	import flash.display.Bitmap;
	import flash.display.BitmapData;

	import spine.atlas.AtlasPage;
	import spine.atlas.AtlasRegion;
	import spine.atlas.TextureLoader;

	public class FlashTextureLoader implements TextureLoader {
		public var bitmapDatas : Object = {};
		public var singleBitmapData : BitmapData;

		/** @param bitmaps A Bitmap or BitmapData for an atlas that has only one page, or for a multi page atlas an object where the
		 * key is the image path and the value is the Bitmap or BitmapData. */
		public function FlashTextureLoader(bitmaps : Object) {
			if (bitmaps is BitmapData) {
				singleBitmapData = BitmapData(bitmaps);
				return;
			}
			if (bitmaps is Bitmap) {
				singleBitmapData = Bitmap(bitmaps).bitmapData;
				return;
			}

			for (var path : * in bitmaps) {
				var object : * = bitmaps[path];
				var bitmapData : BitmapData;
				if (object is BitmapData)
					bitmapData = BitmapData(object);
				else if (object is Bitmap)
					bitmapData = Bitmap(object).bitmapData;
				else
					throw new ArgumentError("Object for path \"" + path + "\" must be a Bitmap or BitmapData: " + object);
				bitmapDatas[path] = bitmapData;
			}
		}

		public function loadPage(page : AtlasPage, path : String) : void {
			var bitmapData : BitmapData = singleBitmapData || bitmapDatas[path];
			if (!bitmapData)
				throw new ArgumentError("BitmapData not found with name: " + path);
			page.rendererObject = bitmapData;
			page.width = bitmapData.width;
			page.height = bitmapData.height;
		}

		public function loadRegion(region : AtlasRegion) : void {
		}

		public function unloadPage(page : AtlasPage) : void {
			BitmapData(page.rendererObject).dispose();
		}
	}
}
