/******************************************************************************
 * Spine Runtimes License Agreement
 * Last updated January 1, 2020. Replaces all prior versions.
 *
 * Copyright (c) 2013-2020, Esoteric Software LLC
 *
 * Integration of the Spine Runtimes into software or otherwise creating
 * derivative works of the Spine Runtimes is permitted under the terms and
 * conditions of Section 2 of the Spine Editor License Agreement:
 * http://esotericsoftware.com/spine-editor-license
 *
 * Otherwise, it is permitted to integrate the Spine Runtimes into software
 * or otherwise create derivative works of the Spine Runtimes (collectively,
 * "Products"), provided that each user of the Products must obtain their own
 * Spine Editor license and redistribution of the Products in any form must
 * include this license and copyright notice.
 *
 * THE SPINE RUNTIMES ARE PROVIDED BY ESOTERIC SOFTWARE LLC "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ESOTERIC SOFTWARE LLC BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES,
 * BUSINESS INTERRUPTION, OR LOSS OF USE, DATA, OR PROFITS) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THE SPINE RUNTIMES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

package com.esotericsoftware.spine;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.backends.lwjgl.LwjglApplication;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureAtlas.AtlasSprite;
import com.badlogic.gdx.utils.ScreenUtils;

public class FrameByFrameTest extends ApplicationAdapter {
	OrthographicCamera camera;
	PolygonSpriteBatch batch;

	TextureAtlas atlas;
	float time;
	Animation<AtlasSprite> walkAnimation, deathAnimation, current;

	public void create () {
		camera = new OrthographicCamera();
		batch = new PolygonSpriteBatch();

		atlas = new TextureAtlas("spineboy/frame-by-frame.atlas");

		walkAnimation = new Animation(1 / 20f, atlas.createSprites("spineboy-pro-walk"));
		walkAnimation.setPlayMode(PlayMode.LOOP);

		deathAnimation = new Animation(1 / 20f, atlas.createSprites("spineboy-pro-death"));

		current = walkAnimation;
	}

	public void render () {
		time += Gdx.graphics.getDeltaTime();
		if (current == deathAnimation && current.isAnimationFinished(time)) {
			current = walkAnimation;
			time = 0;
		}
		if (Gdx.input.justTouched()) {
			current = deathAnimation;
			time = 0;
		}

		AtlasSprite frame = current.getKeyFrame(time);
		float x = Math.round(Gdx.graphics.getWidth() / 2), y = 25;
		int[] origin = frame.getAtlasRegion().findValue("origin");
		x -= origin[0];
		y -= origin[1];
		frame.setPosition(x, y);

		ScreenUtils.clear(0, 0, 0, 0);

		camera.update();
		batch.getProjectionMatrix().set(camera.combined);

		batch.begin();
		frame.draw(batch);
		batch.end();
	}

	public void resize (int width, int height) {
		camera.setToOrtho(false); // Update camera with new size.
	}

	public void dispose () {
		atlas.dispose();
	}

	public static void main (String[] args) throws Exception {
		new LwjglApplication(new FrameByFrameTest());
	}
}
